@@ -1,5 +1,5 @@
 from .body import __init__
 from .track import __init__
-from .modules import __init__
+from .utils import __init__
 from .cross_sections import __init__
 from .particle import __init__
