@@ -1,5 +1,5 @@
 import numpy as np
-from taurunner.modules import units, Callable
+from taurunner.utils import units, Callable
 from scipy.integrate import quad
 
 class Body(object):
@@ -7,7 +7,7 @@ class Body(object):
     def __init__(self,
                  density, 
                  radius: float, 
-                 layer_boundaries: list = None, 
+                 proton_fraction=0.5,
                  name:str = None
                 ):
         r'''
@@ -19,32 +19,32 @@ class Body(object):
                            that takes a radius (0<=r<=1) and returns a float, or a list of such objects.
                            [gr/cm^3]
         radius           : Radius of the body [meters]
-        layer_boundaries : List which tells the boundaries of different denstiy regions.
-                           Must be provided if the density is given as a list, else will be ignored
-        # TODO check if name gets used at all....
         name             : Name to give your object
-        
         '''
 
-        self.radius = radius*units.km
-        self._name  = name
+        self.radius    = radius*units.km
+        self.radius_km = radius
+        self._name     = name
         # Check if body is segmented
         if hasattr(density, '__iter__'):
-            if layer_boundaries is None:
-                raise RuntimeError('You must specify layer boundaries')
-            elif len(layer_boundaries)!=len(density)+1:
-                raise RuntimeError('Density and layer_boundaries must have same length.')
-            else:
-                self._is_layered      = True
-                self._density         = [units.gr/units.cm**3*Callable(obj) for obj in density]
-                self.layer_boundaries = layer_boundaries
+            self._density    = [units.gr/units.cm**3*Callable(tup[0]) for tup in density]
+            layer_boundaries = [tup[1] for tup in density]
+            layer_boundaries.insert(0, 0) # the first layer boundary always has to be 0
+            self.layer_boundaries = np.array(layer_boundaries)
         else:
-            self._is_layered      = False
             self._density         = [units.gr/units.cm**3*Callable(density)]
             self.layer_boundaries = np.array([0.0, 1.0])
+        if hasattr(proton_fraction, '__iter__'):
+            self._pfract      = [Callable(tup[0]) for tup in proton_fraction]
+            player_boundaries = [tup[1] for tup in proton_fraction]
+            player_boundaries.insert(0, 0) # the first layer boundary always has to be 0
+            self.player_boundaries = np.array(player_boundaries)
+        else:
+            self._pfract           = [Callable(proton_fraction)]
+            self.player_boundaries = np.array([0.0, 1.0])
         self._average_densities()
 
-    def get_density(self, r: float) -> float:
+    def get_density(self, r: float, right: bool=False) -> float:
         r'''
         Function to get density at an input radius (0<=r<=1)
 
@@ -58,12 +58,14 @@ class Body(object):
         '''
         if r==1:
             layer_index = -1
+        elif r==0:
+            layer_index = 0
+	
         else:
-            layer_index = np.digitize(r, self.layer_boundaries)-1
+            layer_index = np.digitize(r, self.layer_boundaries, right=right)-1
         current_density = self._density[layer_index]
         return current_density(r)
 
-    # TODO do we need this ?
     def get_average_density(self, r: float) -> float:
         r'''
         Function to return the average density of the layer in which the input radius is.
@@ -91,3 +93,24 @@ class Body(object):
             average_density.append(I[0]/(xf-xi))
 
         self._average_density = average_density
+
+    def get_proton_fraction(self, r: float, right: bool=False) -> float:
+        r'''
+        Function to get density at an input radius (0<=r<=1)
+
+        Params
+        ______
+        r : Radius in units of radius of the body, i.e. center is r=0 and edge is r=1
+
+        Returns
+        _______
+        current_pfraction : Proton fraction of the body at the input radius in natural units [eV^4]
+        '''
+        if r==1:
+            layer_index = -1
+        elif r==0:
+            layer_index = 0
+        else:
+            layer_index = np.digitize(r, self.player_boundaries, right=right)-1
+        current_density = self._pfract[layer_index]
+        return current_density(r)
