@@ -3,14 +3,18 @@
 import os, sys, json
 import proposal as pp
 
-from taurunner.modules import units, cleanup_outdir, sample_powerlaw, is_floatable, make_propagator
-from taurunner.track import Chord
+from taurunner.utils import units, make_propagator
+from taurunner import track
 from taurunner.body import *
 from taurunner.cross_sections import CrossSections
 from taurunner.Casino import *
 from taurunner.particle import Particle
+from taurunner.utils.make_track import make_track
 
 def initialize_parser(): # pragma: no cover
+    r'''
+    Helper function to parse command-line arguments
+    '''
     import argparse
     parser = argparse.ArgumentParser()
     parser.add_argument('-s',
@@ -25,12 +29,17 @@ def initialize_parser(): # pragma: no cover
                         default=0,
                         help='how many events do you want?'
                        )
-    parser.add_argument('-flavor', 
+    parser.add_argument('--flavor', 
                         dest='flavor', 
                         type=int, default=16,
                         help='neutrino flavor (default is nutau): 12 for nue 14 for numu 16 for nutau'
                        )
-
+    parser.add_argument('--track',
+                        dest='track',
+                        type=str,
+                        default='chord',
+                        help='Track type to use. curently only radial and chord trajectories supported.'
+                       )
     # Energy arguments
     parser.add_argument('-e', 
                         dest='energy',
@@ -41,13 +50,13 @@ def initialize_parser(): # pragma: no cover
                        )
     parser.add_argument('--e_min',
                         type=float,
-                        default=1e6,
-                        help='Minimum energy to sample from if doing powerlaw sampling'
+                        default=1e15,
+                        help='Minimum energy to sample from if doing powerlaw sampling (eV)'
                        )
     parser.add_argument('--e_max',
                         type=float,
-                        default=1e9,
-                        help='Minimum energy to sample from if doing powerlaw sampling'
+                        default=1e18,
+                        help='Maximum energy to sample from if doing powerlaw sampling (eV)'
                        )
     
     # Angular arguments
@@ -55,7 +64,7 @@ def initialize_parser(): # pragma: no cover
                         dest='theta',
                         default='',
                         help='nadir angle in degrees if numerical value(0 is through the core).\n\
-                              "Range" if you want to sample from a range of thetas. Use --th_min and --th_max to specify lims.'
+                              "range" if you want to sample from a range of thetas. Use --th_min and --th_max to specify lims.'
                        )
     parser.add_argument('--th_max',
                         dest='th_max',
@@ -70,18 +79,12 @@ def initialize_parser(): # pragma: no cover
                        )
     
     # Saving arguments
-    parser.add_argument('--savedir', 
-                        dest='savedir', 
+    parser.add_argument('--save', 
+                        dest='save', 
                         type=str, 
                         default='', 
                         help="If saving output, provide a path here, if not specified, output will be printed"
                        )
-    parser.add_argument('--prefix',
-                        dest='prefix', 
-                        default='',
-                        type=str,
-                        help='(Optional) argument to give specify name of outdir'
-                       )
 
     # cross section args
     parser.add_argument('--xs',
@@ -133,6 +136,12 @@ def initialize_parser(): # pragma: no cover
                         action='store_true', 
                         help='Do you want to print out debug statments? If so, raise this flag'
                        ) 
+    parser.add_argument('--e_cut',
+                        dest='e_cut',
+                        default=0.0,
+                        type=float,
+                        help='Energy at which to stop the propagation.'
+                       )
 
     args = parser.parse_args()
     return args
@@ -141,9 +150,14 @@ def run_MC(eini: np.ndarray,
            thetas: np.ndarray,
            body: Body, 
            xs: CrossSections, 
-           tracks: dict, 
-           TR_specs: dict, 
-           propagator: pp.Propagator
+           propagator: pp.Propagator,
+           rand: np.random.RandomState = None,
+           no_secondaries: bool = False,
+           flavor: int = 16,
+           no_losses: bool = False,
+           condition=None,
+           depth=0.0,
+           track_type='chord'
           ) -> np.ndarray:
     r'''
     Main simulation code. Propagates an ensemble of initial states and returns the output
@@ -153,8 +167,6 @@ def run_MC(eini: np.ndarray,
     thetas     : array containing the incoming angles of particles to simulate
     body       : taurunner Body object in which to propagate the particles
     xs         : taurunner CrossSections object for interactions
-    tracks     : dictionary whose keys are angles and who values are taurunner Track objects
-    TR_specs   : dictionary specifiying additional TR options
     propagator : PROPOSAL propagator object for charged lepton propagation
     Returns
     _______
@@ -163,51 +175,86 @@ def run_MC(eini: np.ndarray,
              and particle type (PDG convention)
              
     '''
-    output                 = []
-    energies               = list(eini)
-    particleIDs            = np.ones(TR_specs['nevents'], dtype=int)*TR_specs['flavor']
-    rand                   = TR_specs['rand']
-    proposal_lep           = pp.particle.DynamicData(pp.particle.TauMinusDef().particle_type)
-    proposal_lep.position  = pp.Vector3D(0, 0, 0)
-    proposal_lep.direction = pp.Vector3D(0, 0, 1)
-
+    nevents     = len(eini)
+    output      = []
+    energies    = list(eini)
+    particleIDs = np.ones(nevents, dtype=int)*flavor
+    xs_model    = xs.model
+    prev_th     = thetas[0]
+    prev_track  = make_track(prev_th)
+    if rand is None:
+        rand = np.random.RandomState()
+    secondary_basket = []
+    idxx             = []    
+    my_track  = None
+    prv_theta = np.nan
     # Run the algorithm
+
     # All neutrinos are propagated until exiting as tau neutrino or taus.
     # If secondaries are on, then each event has a corresponding secondaries basket
     # which are propagated all at once in the end.
-    for i in range(TR_specs['nevents']):
-        particle = Particle(particleIDs[i], energies[i], thetas[i], 0.0, rand.randint(low=1e9),
-    			    xs, propagator, proposal_lep, not TR_specs['no_secondaries'], TR_specs['no_losses'])
+    for i in range(nevents):
+        cur_theta = thetas[i]
+        cur_e     = energies[i]
+
+        if (cur_theta!=prv_theta and track_type=='chord') or my_track is None: # We need to make a new track
+            my_track = getattr(track, track_type)(theta=cur_theta, depth=depth)
+        particle = Particle(particleIDs[i], 
+                            cur_e, 
+                            0.0 ,
+                            rand, 
+                            xs,
+                            propagator, 
+                            not no_secondaries, 
+                            no_losses
+        )
         
-        my_track = tracks[thetas[i]]
-        out = Propagate(particle, my_track, body)
+        out      = Propagate(particle, my_track, body, condition=condition)
     
         if (out.survived==False):
-            #these muons were absorbed. we record them in the output with outgoing energy 0
-            print('this is weird')
-            output.append((energies[i], 0., thetas[i], out.nCC, out.nNC, out.ID))
-            del out
+            #this muon/electron was absorbed. we record it in the output with outgoing energy 0
+            output.append((cur_e, 0., cur_theta, out.nCC, out.nNC, out.ID, i, out.position))
         else:
-            output.append((energies[i], float(out.energy), thetas[i], out.nCC, out.nNC, out.ID))
-        if not TR_specs['no_secondaries']:
-            basket = out.basket
-            for sec in basket:
-                sec_particle = Particle(sec['ID'], sec['energy'], thetas[i], sec['position'], rand.randint(low=1e9),
-                                        xs=xs, proposal_propagator=None, proposal_lep=None, secondaries=False, no_losses=True)
-                sec_out      = Propagate(sec_particle, my_track, body)
-                if(not sec_out.survived):
-                    output.append((sec_out.energy, 0.0, thetas[i], sec_out.nCC, sec_out.nNC, -sec_out.ID))
-                else:
-                    output.append((sec_out.initial_energy, sec_out.energy, thetas[i], 
-    				                 sec_out.nCC, sec_out.nNC, -sec_out.ID))
-                del sec_particle
-            del out.basket
-            del basket
-        del out     
-        
-    output = np.array(output, dtype = [('Eini', float), ('Eout',float), ('Theta', float), ('nCC', int), ('nNC', int), ('PDG_Encoding', int)])
+            #this particle escaped
+            output.append((cur_e, float(out.energy), cur_theta, out.nCC, out.nNC, out.ID, i, out.position))
+        if not no_secondaries:
+            #store secondaries to propagate later
+            secondary_basket.append(np.asarray(out.basket))
+            #keep track of parent
+            idxx = np.hstack([idxx, [i for _ in out.basket]])
+        prv_theta = cur_theta
+        del out
+        del particle
+    idxx = np.asarray(idxx).astype(np.int32)
+    if not no_secondaries:    
+        #make muon propagator
+        secondary_basket = np.concatenate(secondary_basket)
+        sec_prop         = {ID:make_propagator(ID, body, xs_model) for ID in [-12, -14]}
+        for sec, i in zip(secondary_basket, idxx):
+            cur_theta = thetas[i]
+            if cur_theta!=prv_theta and track_type=='chord': # We need to make a new track
+                my_track = getattr(track, 'chord')(theta=cur_theta, depth=depth)
+            sec_particle = Particle(
+                                    sec['ID'], 
+                                    sec['energy'],
+                                    sec['position'],
+                                    rand,
+                                    xs=xs, 
+                                    proposal_propagator=sec_prop[sec['ID']], 
+                                    secondaries=False, 
+                                    no_losses=False
+                                   )
+            sec_out = Propagate(sec_particle, my_track, body, condition=condition)
+            if(not sec_out.survived):
+                output.append((sec_out.initial_energy, 0.0, cur_theta, sec_out.nCC, sec_out.nNC, sec_out.ID, i, sec_out.position))
+            else:
+                output.append((sec_out.initial_energy, sec_out.energy, cur_theta, 
+    	    		                 sec_out.nCC, sec_out.nNC, sec_out.ID, i, sec_out.position))
+            prv_theta = cur_theta
+            del sec_particle
+            del sec_out
+    output = np.array(output, dtype = [('Eini', float), ('Eout',float), ('Theta', float), ('nCC', int), ('nNC', int), ('PDG_Encoding', int), ('event_ID', int), ('final_position', float)])
     output['Theta'] = np.degrees(output['Theta'])
-                       
     return output
 
 if __name__ == "__main__": # pragma: no cover
@@ -217,13 +264,14 @@ if __name__ == "__main__": # pragma: no cover
     TR_specs['seed']           = args.seed
     TR_specs['nevents']        = args.nevents
     TR_specs['flavor']         = args.flavor
+    TR_specs['track']          = args.track
     TR_specs['energy']         = args.energy
     TR_specs['e_min']          = args.e_min
     TR_specs['e_max']          = args.e_max
     TR_specs['theta']          = args.theta
     TR_specs['th_min']         = args.th_min
     TR_specs['th_max']         = args.th_max
-    TR_specs['base_savedir']   = args.savedir
+    TR_specs['save']           = args.save
     TR_specs['water']          = args.water
     TR_specs['xs_model']       = args.xs_model
     TR_specs['no_losses']      = args.no_losses
@@ -231,53 +279,57 @@ if __name__ == "__main__": # pragma: no cover
     TR_specs['radius']         = args.radius
     TR_specs['depth']          = args.depth
     TR_specs['no_secondaries'] = args.no_secondaries
-    TR_specs['prefix']         = args.prefix
     TR_specs['debug']          = ''
-    rand = np.random.RandomState(TR_specs['seed'])
-    TR_specs['rand'] = rand
+
     if TR_specs['nevents']<=0:
         raise ValueError("We need to simulate at least one event, c'mon y'all")
 
     # Set up outdir and make seed if not provided
-    if TR_specs['base_savedir']:
-        if not os.path.isdir(TR_specs['base_savedir']):
-            raise ValueError('Savedir %s does not exist' % TR_specs['base_savedir'])
-        from taurunner.modules import setup_outdir
-        TR_specs = setup_outdir(TR_specs)
+    if TR_specs['save']:
+        savedir = '/'.join(TR_specs['save'].split('/')[:-1])
+        if not os.path.isdir(savedir):
+            raise ValueError('Savedir %s does not exist' % TR_specs['save'])
     
     # Set up a random state
     rand = np.random.RandomState(TR_specs['seed'])
-    TR_specs['rand'] = rand
-
-    # Make an array of injected energies
-    from taurunner.modules import make_initial_e
-    eini = make_initial_e(TR_specs, rand=rand)
-
-    # Make an array of injected incident angles
-    from taurunner.modules import make_initial_thetas
-    thetas = make_initial_thetas(TR_specs, rand=rand)
 
     # Make body
     # TODO Make this not suck. i.e. make construct body more comprehensive
-    from taurunner.modules import construct_body
+    from taurunner.utils import construct_body
     body = construct_body(TR_specs)
 
-    # Premake all necessary tracks in case of redundancies
-    # TODO Make it so that you can pass radial tracks too
-    tracks  = {theta:Chord(theta=theta, depth=TR_specs['depth']/body.radius) for theta in set(thetas)}
+    # Make an array of injected energies
+    from taurunner.utils.make_initial_e import make_initial_e
+    eini = make_initial_e(TR_specs['nevents'], TR_specs['energy'],
+                          e_min=TR_specs['e_min'], e_max=TR_specs['e_max'], rand=rand)
 
+    # Make an array of injected incident angles
+    from taurunner.utils.make_initial_thetas import make_initial_thetas
+    if(TR_specs['theta']=='range'):
+        theta = (TR_specs['th_min'], TR_specs['th_max'])
+    else:
+        theta = TR_specs['theta']
+    thetas = make_initial_thetas(TR_specs['nevents'], theta, rand=rand, track_type=TR_specs['track'])
+    sorter = np.argsort(thetas)
+    thetas = thetas[sorter]
+    
     xs = CrossSections(TR_specs['xs_model'])
 
-    prop = make_propagator(body, xs_model=xs.model)
+    prop = make_propagator(TR_specs['flavor'], body, TR_specs['xs_model'])
+    if args.e_cut:
+        condition = lambda particle: (particle.energy <= args.e_cut*units.GeV and abs(particle.ID) in [12,14,16])
+    else:
+        condition = None
 
-    result = run_MC(eini, thetas, body, xs, tracks, TR_specs, prop)
+    result = run_MC(eini, thetas, body, xs, prop, rand=rand,
+                    no_secondaries=TR_specs['no_secondaries'], no_losses=TR_specs['no_losses'],
+                    flavor=TR_specs['flavor'], condition=condition, depth=TR_specs['depth'], track_type=TR_specs['track'])
 
-    # TODO simplify this
-    if TR_specs['base_savedir']:
-        base_fname = '%s/%s/%s' % (TR_specs['base_savedir'], TR_specs['prefix'], TR_specs['prefix'])
-        np.save(base_fname+'.npy', result)
-        with open(base_fname + '.json', 'w') as f:
-            TR_specs.pop('rand')
+    if TR_specs['save']:
+        if '.npy' not in TR_specs['save']:
+            TR_specs['save'] += '.npy'
+        np.save(TR_specs['save'], result)
+        with open(TR_specs['save'].replace('npy', 'json'), 'w') as f:
             json.dump(TR_specs, f)
         if TR_specs['debug']:
             print(message)
