@@ -3,56 +3,49 @@ import proposal as pp
 from importlib.resources import path
 
 import taurunner
-from taurunner.modules import units
+from taurunner.utils import units
+from taurunner.cross_sections import CrossSections
 from .utils import *
-from taurunner.resources import secondaries_splines
 
-# TODO move this into utils
-# load secondary cdfs
-with path(secondaries_splines, 'antinue_cdf.npy') as p:
-        nue_path = str(p)
-with path(secondaries_splines, 'antinumu_cdf.npy') as p:
-        numu_path = str(p)
-
-antinue_cdf = np.load(nue_path)
-antinumu_cdf = np.load(numu_path)
-bins = list(np.logspace(-5,0,101))[:-1] # bins for the secondary splines
-
-def get_sample(u, cdf):
-    spl_cdf = iuvs(bins, cdf)
-    # check if random sample u is in the range where the spline is defined
-    try:
-        return (iuvs(bins,cdf-u).roots())[0]
-    except:
-        if u <= np.min(spl_cdf(bins)): 
-            return 1e-3
-        elif u == np.max(spl_cdf(bins)):
-            return 1
+from proposal import Propagator
+from numpy.random import RandomState
+import warnings
 
 proton_mass = ((0.9382720813+0.9395654133)/2.)*units.GeV
 
+ID_2_name = {13:'MuMinusDef', -13:'MuPlusDef', 
+             15:'TauMinusDef', -15:'TauPlusDef'}
+EMIN = 1e9 # minimum energy allowed in the splines
 #Particle object. 
 class Particle(object):
     r'''
     This is the class that contains all relevant 
     particle information stored in an object.
     '''
-    def __init__(self, ID, energy, incoming_angle, position, seed,
-                   xs, proposal_propagator, proposal_lep, secondaries, no_losses):
+    def __init__(self, 
+                 ID:                  int, 
+                 energy:              float, 
+                 position:            float, 
+                 rand:                RandomState,
+                 xs:                  CrossSections, 
+                 proposal_propagator: Propagator,
+                 secondaries:         bool, 
+                 no_losses:           bool
+                ):
         r'''
         Class initializer. This function sets all initial conditions based 
         on the particle's incoming angle, energy, ID, and position.
 
-        Parameters
-        ----------
-        ID:    int
-            The particle ID as defined in the PDG MC encoding.
-        energy:         float
-            Initial energy in eV.
-        incoming_angle: float
-            The incident angle with respect to nadir in radians.
-        seed:           int
-            Seed corresponding to the random number generator.
+        Params
+        ------
+        ID                  : PDG particle identifier
+        energy              : Initial energy of the particle [eV]
+        position            : Affine paramter describing the distance along the track of the particle (0<x<1)
+        rand                : numpy random number generator
+        xs                  : TauRunner CrossSections object
+        proposal_propagator : PROPOSAL propagator object for moving charged lepton
+        secondaries         : Boolean telling whether to include secondary (mu and e) neutrinos from tau decay
+        no_losses           : Boolean to turn off charged lepton losses
         '''
         #Set Initial Values
         self.ID              = ID
@@ -67,24 +60,27 @@ class Particle(object):
         self.nCC             = 0
         self.nNC             = 0
         self.ntdecay         = 0
-        self.rand            = np.random.RandomState(seed=seed)
+        self.rand            = rand
         self.xs              = xs
         self.xs_model        = xs.model
         self.propagator      = proposal_propagator
         self.losses          = not no_losses
-        self.lep             = proposal_lep
+        if ID > 0:
+            self.nutype = 'nu'
+        elif ID < 0:
+            self.nutype = 'nubar'
         
     def SetParticleProperties(self):
         r'''
         Sets particle properties, either when initializing or after an interaction.
         '''
-        if self.ID in [12, 14, 16]:
+        if np.abs(self.ID) in [12, 14, 16]:
             self.mass = 0.0          #this is not true.. and it seems to have caused quite the stir.
-            self.lifetime = np.inf   #this is unknown
-        if self.ID == 15:
+            self.lifetime = np.inf   #this is unclear
+        if np.abs(self.ID) == 15:
             self.mass = 1.776*units.GeV
             self.lifetime = 2.9e-13*units.sec
-        if self.ID == 13:
+        if np.abs(self.ID) == 13:
             self.mass = 0.105*units.GeV
             self.livetime = 2.2e-6*units.sec
 
@@ -139,7 +135,7 @@ class Particle(object):
         return(1./(1./self.GetInteractionDepth(interaction='NC')
                + 1./self.GetInteractionDepth(interaction='CC')))
 
-    def GetInteractionDepth(self, interaction):
+    def GetInteractionDepth(self, interaction, proton_fraction=0.5):
         r'''
         Calculates the mean column depth to interaction.
 
@@ -152,115 +148,106 @@ class Particle(object):
         Interaction depth: float
             mean column depth to interaction in natural units
         '''
-        if self.ID in [12, 14, 16]:
-            return proton_mass/(self.xs.TotalNeutrinoCrossSection(self.energy, interaction = interaction))
-        if self.ID == 15:
+        if np.abs(self.ID) in [12, 14, 16]:
+            return proton_mass/(self.xs.total_cross_section(
+                                                            self.energy, 
+                                                            self.nutype, 
+                                                            interaction,
+                                                            proton_fraction=proton_fraction
+                                                           )
+                               )
+        if np.abs(self.ID) == 15:
             raise ValueError("Tau interaction length should never be sampled.")
 
     def GetInteractionProbability(self,ddepth,interaction):
         return 1.-np.exp(-ddepth/self.GetInteractionDepth(interaction))
 
     def Decay(self):
-        if self.ID in [12, 14, 16]:
+        if np.abs(self.ID) in [12, 14, 16]:
             raise ValueError("no neutrino decays.. yet")
-        if self.ID == 15:
+        if np.abs(self.ID) == 15:
             if self.secondaries:
                 # sample branching ratio of tau leptonic decay
                 p0 = self.rand.uniform(0,1)
                 if p0 < .18:
                     # sample energy of secondary antinumu
-                    sample = get_sample(self.rand.uniform(0,1), antinumu_cdf)
+                    sample = SampleSecondariesEnergyFraction(self.rand.uniform(0,1), antinumu_cdf)
                     enu = sample*self.energy
                     # add secondary to basket, prepare propagation
-                    self.basket.append({"ID" : 14, "position" : self.position, "energy" : enu})
+                    self.basket.append({"ID" : -np.sign(self.ID)*14, "position" : self.position, "energy" : enu})
                 elif p0 > .18 and p0 < .36:
                     # sample energy of secondary antinue
-                    sample = get_sample(self.rand.uniform(0,1), antinue_cdf)
+                    sample = SampleSecondariesEnergyFraction(self.rand.uniform(0,1), antinue_cdf)
                     enu = sample*self.energy
                     # add secondary to basket, prepare propagation
-                    self.basket.append({"ID" : 12,  "position" : self.position, "energy" : enu})
+                    self.basket.append({"ID" : -np.sign(self.ID)*12,  "position" : self.position, "energy" : enu})
             self.energy = self.energy*self.rand.choice(TauDecayFractions, p=TauDecayWeights)
-            self.ID = 16
+            self.ID = np.sign(self.ID)*16
             self.SetParticleProperties()
             return
-        if self.ID == 13:
-            #print('uuuuh???')
+        if np.abs(self.ID) in [11, 13]:
             self.survived=False
 
-    def PropagateChargedLepton(self, body, track): #description is wrong that should be fixed
+    def PropagateChargedLepton(self, body, track):
         r'''
-        propagate taus/mus through medium
+        Propagate taus/mus with PROPOSAL along 'track' through 'body'
         Parameters
         ----------
-        objects: list
-            List of CasinoEvents that need to have tau losses sampled stochastically.
-        xs: str 
+        body: str 
             Cross section model to use for the photohadronic losses
-        losses: bool
+        track: bool
             This can be set to False to turn off energy losses. In this case, the particle decays at rest.
-        Returns
-        -------
-        objects: list
-            List of CasinoEvents after losses are calculated
         '''
-        #if self.ID==13:
-        #  flavor='mu'
-        #  lep = pp.particle.DynamicData(pp.particle.MuMinusDef().particle_type)
-        #elif self.ID==15: # pragma: no cover
-        #  flavor='tau'
-        #  lep = pp.particle.DynamicData(pp.particle.TauMinusDef().particle_type)
-        lep = self.lep #[self.ID]
-        current_km_dist  = track.x_to_d(self.position)*body.radius/units.km
-        total_dist       = track.x_to_d(1.-self.position)*body.radius/units.km
-        current_density  = body.get_average_density(track.x_to_r(self.position))
-        dist_to_prop     = 1e3*(total_dist - current_km_dist)
-
-        if(not self.losses): #easy
+        if(np.logical_or(not self.losses, np.abs(self.ID) in [11, 12])):
             return
-        #elif(self.energy/units.GeV <= 1e6):
-        #    self.chargedposition = ((self.energy/units.GeV/1e6)*50.)/1e3
-        else: # pragma: no cover
-            lep_length  = []
-            en_at_decay = []
-            #need to add support to propagate without decay here (fixed distance propagation)
-            lep.energy     = 1e3*self.energy/units.GeV
-            rad = body.radius/units.km*1e5
-            phi            = 2.*track.theta
-            pos_vec   = pp.Vector3D(rad*np.sin(phi)*(1. - self.position), 0, rad*((1. - track.depth + np.cos(phi))*self.position - np.cos(phi)))
-            direction = [-np.sin(phi), 0., np.cos(phi) + (1. - track.depth)]
-            norm = np.linalg.norm(direction)
-            lep.position   = pos_vec
-            lep.direction  = pp.Vector3D(direction[0]/norm, 0., direction[2]/norm)
-            sec            = self.propagator.propagate(lep) #, dist_to_prop)
-            particles      = sec.particles
-            final_vec      = (sec.position[-1] - pos_vec)
-            lep_length     = final_vec.magnitude() / 1e5
-            decay_products = [p for i,p in zip(range(max(len(particles)-3,0),len(particles)), particles[-3:]) if int(p.type) <= 1000000001]
-            en_at_decay    = np.sum([p.energy for p in decay_products])
-            self.energy    = en_at_decay*units.GeV/1e3
-            self.chargedposition  = lep_length
-        return
+        lep              = pp.particle.DynamicData(getattr(pp.particle, ID_2_name[self.ID])().particle_type)
+        total_dist       = track.x_to_d(1.-self.position)*body.radius/units.km
+        if(np.logical_and(np.abs(self.ID) in [13, 14], total_dist > 100.)):  #muons farther than 100km will not make it
+             return
 
-    def Interact(self, interaction, body=None, track=None): #  dist_to_prop=None, current_density=None):
-        if self.ID in [12, 14, 16]:
+        lep_length  = []
+        en_at_decay = []
+        lep.energy     = self.energy/units.MeV
+        pos_vec        = track.x_to_pp_pos(self.position, body.radius/units.cm) # radius in cm
+        dir_vec        = track.x_to_pp_dir(self.position)
+        lep.position   = pos_vec
+        lep.direction  = dir_vec
+        #propagate
+        sec            = self.propagator.propagate(lep, total_dist*1e5) #, dist_to_prop)
+        particles      = sec.particles
+        #update particle info
+        final_vec      = (sec.position[-1] - pos_vec)
+        lep_length     = final_vec.magnitude() / 1e5
+        decay_products = [p for i,p in zip(range(max(len(particles)-3,0),len(particles)), particles[-3:]) if int(p.type) <= 1000000001]
+        en_at_decay    = np.sum([p.energy for p in decay_products])
+        if(en_at_decay==0):       #particle reached the border before decaying
+            self.energy           = particles[-1].parent_particle_energy*units.MeV
+            self.chargedposition  = float(np.ceil(lep_length))
+        else:                     #particle decayed before reaching the border
+            self.energy           = en_at_decay*units.MeV
+            self.chargedposition  = lep_length
+        return sec
+
+    def Interact(self, interaction, body=None, track=None, proton_fraction=0.5): #  dist_to_prop=None, current_density=None):
+        if np.abs(self.ID) in [12, 14, 16]:
             #Sample energy lost from differential distributions
-            NeutrinoInteractionWeights = self.xs.DifferentialOutGoingLeptonDistribution(
-                self.energy/units.GeV,
-                self.energy*NeutrinoDifferentialEnergyFractions/units.GeV,
-                interaction)
+            NeutrinoInteractionWeights = self.xs.differential_cross_section(self.energy,
+                                                                            NeutrinoDifferentialEnergyFractions,
+                                                                            self.nutype,
+                                                                            interaction,
+                                                                            proton_fraction=proton_fraction
+                                                    )
             NeutrinoInteractionWeights = np.divide(NeutrinoInteractionWeights, 
                                                    np.sum(NeutrinoInteractionWeights))
-            self.energy = self.energy*self.rand.choice(NeutrinoDifferentialEnergyFractions,
+            z_choice = self.rand.choice(NeutrinoDifferentialEnergyFractions,
                                                        p=NeutrinoInteractionWeights)
+            self.energy = z_choice*(self.energy-EMIN)+EMIN
 
             if interaction == 'CC':
                 #make a charged particle
                 self.nCC += 1
-                if(self.ID==16):
-                    self.ID = 15
-                #elif(self.ID==14):
-                #    self.ID = 13
-                elif(self.ID in [12, 14]):
+                self.ID = np.sign(self.ID)*(np.abs(self.ID)-1)
+                if(np.abs(self.ID)==11): #electrons have no chance
                     self.survived=False
                     return
                 self.SetParticleProperties()
